@@ -1,17 +1,15 @@
 /*
- * password.h
+ *  password.h
  *
- * Copyright (C) 2007, 2008, 2009, 2013
+ *  Password Utilities for AES Crypt
+ *  Copyright (C) 2022
+ *  Paul E. Jones <paulej@packetizer.com>
  *
- * This software is licensed as "freeware."  Permission to distribute
- * this software in source and binary forms is hereby granted without a
- * fee.  THIS SOFTWARE IS PROVIDED 'AS IS' AND WITHOUT ANY EXPRESSED OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
- * THE AUTHOR SHALL NOT BE HELD LIABLE FOR ANY DAMAGES RESULTING FROM
- * THE USE OF THIS SOFTWARE, EITHER DIRECTLY OR INDIRECTLY, INCLUDING,
- * BUT NOT LIMITED TO, LOSS OF DATA OR DATA BEING RENDERED INACCURATE.
+ *  Description:
+ *      Password utilities for AES Crypt.
  *
+ *  Portability Issues:
+ *      None.
  */
 
 #ifndef AESCRYPT_PASSWORD_H
@@ -22,9 +20,8 @@
 
 typedef enum {UNINIT, DEC, ENC} encryptmode_t;
 
-/*
- * Error codes for read_password function.
- */
+// Error codes for read_password function.
+#define AESCRYPT_READPWD_NONE         0
 #define AESCRYPT_READPWD_FOPEN       -1
 #define AESCRYPT_READPWD_FILENO      -2
 #define AESCRYPT_READPWD_TCGETATTR   -3
@@ -33,18 +30,12 @@ typedef enum {UNINIT, DEC, ENC} encryptmode_t;
 #define AESCRYPT_READPWD_TOOLONG     -6
 #define AESCRYPT_READPWD_NOMATCH     -7
 
-/*
- * Function Prototypes
- */
+// Function prototypes
+const char* read_password_error(int error);
+int read_password(unsigned char *buffer, encryptmode_t mode);
 int passwd_to_utf16(unsigned char *in_passwd,
                     int length,
                     int max_length,
                     unsigned char *out_passwd);
 
-const char* read_password_error(int error);
-
-int read_password(unsigned char* buffer,
-                  encryptmode_t mode);
-
-
 #endif // AESCRYPT_PASSWORD_H
